#pragma once

#include "Context.h"
#include "Rescaler.h"

#include <iostream>

namespace WorldMachine {
	class ContextOverride: public PacketContext {
	public:
		ContextOverride();

		enum SCALE_METHOD { SCALED, FIXED };

		bool Load(std::istream &in);
		bool Save(std::ostream &out);

		bool isEnabled() { return enabled; };
		void setEnabled(bool _e) { enabled = _e; };

		SCALE_METHOD getScaleMethod() { return method; };
		void setScaleMethod(SCALE_METHOD s) { method = s;};
		bool isFixedRes() { return method == FIXED; };

		Rescale::RescaleInfo &getRescaleInfo() { return rescale_info; };
		void setRescaleInfo(Rescale::RescaleInfo t) { rescale_info = t; };


		void setScalar(float sc) { scalar = sc; };
		float getScalar() { return scalar; };

		/// This currently modifies the destcontext. But really, we should change this so that we instead inherit the contents of the specified if we're enabled
		void overrideContext( BuildContext &destcontext );

		/// Return what the given resolution would be if it was modified by the context override. This means either will return a local or scaled result
		int getOverrideResolution(int in_res);

	private:

		/// Private version that also considers if this is a tiled build or not. This version is used when actually overriding a context.
		int getOverrideResolution(BuildContext &destcontext);


		SCALE_METHOD method;
		float scalar;
		Rescale::RescaleInfo rescale_info;
		bool enabled;
	};
}

