#pragma once

#include "HField.h"

/** Simple wrapper to provide a default value if no heightfield is present.
	Provide a scalar value and a heightfield. If there is no heightfield, it will return the scalar alone.
	Otherwise, it will return the sampled heightfield value.
	@ingroup data
	*/
class InputWrap
{
public:
	InputWrap(HFPointer hf, const float default_val = 0.0f);

	void SetDefault(float _val) { val = _val; };
	inline float value(const Coord &loc) const;
	inline float operator[](const Coord &loc) const;
	inline float operator[](const size_t index) const;
protected:
	HFPointer map;
	float val;
};

inline InputWrap::InputWrap(HFPointer hf, const float default_val) {
	map = hf;
	val = default_val;
};

float InputWrap::value(const Coord &loc) const {
	if (map)
		return map->value(loc);
	else
		return val;
};

float InputWrap::operator[](const Coord &loc) const {
	if (map)
		return map->value(loc);
	else
		return val;
};

float InputWrap::operator[](const size_t index) const {
	if (map)
		return (*map)[index];
	else
		return val;
};

/** Multiplicative wrap around a heightfield/parameter value combo.
	Provide a scalar value and a heightfield. If there is no heightfield, it will return the scalar alone.
	Otherwise, it will multiply the heightfield sampled at the given location by the scalar and return that.
*/
class InputWrapMult : public InputWrap {
public:
	InputWrapMult(HFPointer hf, const float mult_val) : InputWrap(hf, mult_val) { };
	inline float value(const Coord &loc) const;
	inline float operator[](const Coord &loc) const;
	inline float operator[](const size_t index) const;
};

float InputWrapMult::value(const Coord &loc) const {
	if (map)
		return map->value(loc) * val;
	else
		return val;
};

float InputWrapMult::operator[](const Coord &loc) const {
	if (map)
		return map->value(loc) * val;
	else
		return val;
};

float InputWrapMult::operator[](const size_t index) const {
	if (map)
		return (*map)[index] * val;
	else
		return val;
};

