#ifndef _PLUGIN_H
#define _PLUGIN_H
/** @file PluginHeader.h
	@ingroup devices

Contains data structures and details needed to implement a new plugin DLL for World Machine
*/
#include "Globals.h"

/// Your DLL will export this value. It allows the WM framework to only load plugins
/// that were built against the current headers
const int WM_PLUGIN_HEADER_VERSION = 103;		// Current version, compared to the below:
												// 1: Initial Public v1.0 Release
												// 2: WM 1.1 / WM1.25 Release
												// 3: WM Pro / WM2.0 Release
												// 4: WM 2.2 Release
												// 5: WM 2.3 B1
												// 6: WM 2.3 B3
												// 7: WM 2.3 B3part2 -- incremented because we renamed standard plug ins.
												// 8: WM 2.3 RC1  -- incremented because of core changes.
												// 9: WM 2.3 Final -- incremented for release
												// 10: WM 2.3.3dev1 Notification APIs added
												// 11: WM 2.3.3dev3 Notification APIs wrapped up
												// 12: WM 2.3.4
												// 13: WM 2.9Dev branch Release 1
												// 14: internal release to specific licensee
												// 15: WM 2.3.6
												// 16: WM 2.3.6.1 (changed output class)
												// 102: WM 2.9.0.2 Dev Branch 2
												// 103: WM 2.9.0.7 Dev Branch -- incremented because of additional WMINTERFACE in DeviceWorld.h

/// Each device in a plugin has a DevPluginStruc entry. See the example files for details of usage.
struct DevPluginStruc {
	DevPluginStruc() { deprecated = false; type = 0; name[0] = 0; };
	DEV_DATA lifedata;			// Data & Ptrs to life management functions
	char	name[64];			// Name of this plugin
	int		type;				// What category does this device fit into? (See Dev Type Constants Below)
	int		datatype;			// What type of data does this work on (See WMPacket constants). Can be multiple | together
	bool	deprecated;			// Set to true if this device shouldnt show up in the listing anymore
	char	reserved[64];		// room for future expansion...
};

///@name Required Plugin Functions
//@{
//***********************************************************************//
/// (Required) All plugins need to implement this function; WM will not allow load without it
typedef void (*SetupGlobalAccessPtr)(WorldMachine::WMCoreInterface *global);
/// (Required) DLL reports version of WM core that the plugin is built against.
///            Required for WM to do version-error checking of plugins. This function should simply return WM_PLUGIN_HEADER_VERSION
typedef	int  (*GetHeaderVersionPtr)();
/// (Optional) Plugins should implement this if one or more of the devices are registered-only
typedef int (*RegOnlyPtr)();

// *********************************************************************************
/// C++ iteration function to implement for Device Plugins
typedef	bool (*GetDevPluginPtr)(const int index, DevPluginStruc *result);
// *********************************************************************************
//@}

/// @name Device Type Constants
//@{
// Heightfield devices
const int TYPE_GENERATOR = 1;
const int TYPE_OUTPUT = 2;
const int TYPE_COMBINER = 3;
const int TYPE_FILTER = 4;
const int TYPE_NATURAL = 5;
const int TYPE_SELECTOR = 6;
const int TYPE_UTILITY = 7;
const int TYPE_MISC = 8;
const int TYPE_CONVERTER = 9;
// Param/Scalar devices
const int TYPE_SCALAR_OPERATOR = 16;
const int TYPE_PARAM		   = 17;
//@}

#endif